@@ -36,8 +36,15 @@ int main(int, char**)
   Glib::ThreadPool pool (10);
 
   for(char c = 'a'; c <= 'z'; ++c)
-    pool.push(sigc::bind(sigc::ptr_fun(&print_char), c));
-
+  {
+// The AIX xlC compiler gives a linker error, even with the bind<1> hint.
+// See the comment in dispatcher.cc
+// Obviously this example will then be useless on AIX, but at least the build will succeed so people can install the library.
+#if !defined(_AIX)
+    pool.push(sigc::bind<1>(sigc::ptr_fun(&print_char), c));
+#endif
+  }
+  
   pool.shutdown();
 
   std::cout << std::endl;
