@@ -2,7 +2,7 @@
 #ifndef _GLIBMM_USTRING_H
 #define _GLIBMM_USTRING_H
 
-/* $Id: ustring.h,v 1.4 2004/04/11 14:07:06 murrayc Exp $ */
+/* $Id: ustring.h,v 1.7.2.2 2005/01/21 20:12:50 murrayc Exp $ */
 
 /* Copyright (C) 2002 The gtkmm Development Team
  *
@@ -21,6 +21,7 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <glib/gmacros.h>
 #include <glib/gunicode.h>
 #include <glibmm/unicode.h>
 
@@ -133,7 +134,7 @@ private:
  * but it might be useful as utility function if you prefer using
  * std::string even for UTF-8 encoding.
  */
-gunichar get_unichar_from_std_iterator(std::string::const_iterator pos);
+gunichar get_unichar_from_std_iterator(std::string::const_iterator pos) G_GNUC_PURE;
 
 
 /** Glib::ustring has much the same interface as std::string, but contains
@@ -231,28 +232,77 @@ public:
 
 #endif /* GLIBMM_HAVE_SUN_REVERSE_ITERATOR */
 
+#ifdef GLIBMM_HAVE_ALLOWS_STATIC_INLINE_NPOS
   static const size_type npos = std::string::npos;
+#else
+  //The IRIX MipsPro compiler says "The indicated constant value is not known",
+  //so we need to initalize the static member data elsewhere.
+  static const size_type npos;
+#endif
 
+  /*! Default constructor, which creates an empty string.
+   */
   ustring();
+  
   ~ustring();
 
+  /*! Construct a ustring as a copy of another ustring.
+   * @param other A source string.
+   */
   ustring(const ustring& other);
+
+  /*! Assign the value of another string to this string.
+   * @param other A source string.
+   */ 
   ustring& operator=(const ustring& other);
 
+  /*! Swap contents with another string.
+   * @param other String to swap with.
+   */
   void swap(ustring& other);
 
-//! @name Create new strings.
-//! @{
-
+  /*! Construct a ustring as a copy of another std::string.
+   * @param src A source <tt>std::string</tt> containing text encoded as UTF-8.
+   */
   ustring(const std::string& src);
+
+  /*! Construct a ustring as a copy of a substring.
+   * @param src %Source ustring.
+   * @param i Index of first character to copy from.
+   * @param n Number of characters to copy (defaults to copying the remainder).
+   */
   ustring(const ustring& src, size_type i, size_type n=npos);
+
+  /*! Construct a ustring as a partial copy of a C string.
+   * @param src %Source C string encoded as UTF-8.
+   * @param n Number of characters to copy.
+   */
   ustring(const char* src, size_type n);
+
+  /*! Construct a ustring as a copy of a C string.
+   * @param src %Source C string encoded as UTF-8.
+   */
   ustring(const char* src);
+
+  /*! Construct a ustring as multiple characters.
+   * @param n Number of characters.
+   * @param uc UCS-4 code point to use.
+   */  
   ustring(size_type n, gunichar uc);
+
+  /*! Construct a ustring as multiple characters.
+   * @param n Number of characters.
+   * @param c ASCII character to use.
+   */  
   ustring(size_type n, char c);
+
+  /*! Construct a ustring as a copy of a range.
+   * @param pbegin Start of range.
+   * @param pend End of range.
+   */  
   template <class In> ustring(In pbegin, In pend);
 
-//! @}
+
 //! @name Assign new contents.
 //! @{
 
@@ -500,11 +550,11 @@ public:
 //! @{
 
   /*! Returns a new UTF-8 string with all characters characters converted to
-   * their uppercase equivalent, while honoring the current locale.  The
+   * their lowercase equivalent, while honoring the current locale.  The
    * resulting string may change in the number of bytes as well as in the
    * number of characters.  For instance, the German sharp&nbsp;s
-   * <tt>"&szlig;"</tt> will be replaced by two characters <tt>"SS"</tt>
-   * because there is no capital <tt>"&szlig;"</tt>.
+   * <tt>&quot;&szlig;&quot;</tt> will be replaced by two characters
+   * <tt>"SS"</tt> because there is no capital <tt>&quot;&szlig;&quot;</tt>.
    */
   ustring uppercase() const;
 
@@ -534,7 +584,23 @@ private:
   template <class In, class ValueType = typename Glib::IteratorTraits<In>::value_type>
 #endif
   struct SequenceToString;
+  
+  //The Tru64 compiler needs these partial specializations to be declared here,
+  //as well as defined later. That's probably correct. murrayc.
+  template <class In>
+  struct SequenceToString<In, char>;
 
+  template <class In>
+  struct SequenceToString<In, gunichar>;
+  
+  /*
+  template <>
+  struct ustring::SequenceToString<Glib::ustring::iterator, gunichar>;
+
+  template <>
+  struct ustring::SequenceToString<Glib::ustring::const_iterator, gunichar>;
+  */
+  
 #endif /* DOXYGEN_SHOULD_SKIP_THIS */
 
   std::string string_;
@@ -548,25 +614,25 @@ struct ustring::SequenceToString
 {};
 
 template <class In>
-struct ustring::SequenceToString<In,char> : public std::string
+struct ustring::SequenceToString<In, char> : public std::string
 {
   SequenceToString(In pbegin, In pend);
 };
 
 template <class In>
-struct ustring::SequenceToString<In,gunichar> : public std::string
+struct ustring::SequenceToString<In, gunichar> : public std::string
 {
   SequenceToString(In pbegin, In pend);
 };
 
 template <>
-struct ustring::SequenceToString<Glib::ustring::iterator,gunichar> : public std::string
+struct ustring::SequenceToString<Glib::ustring::iterator, gunichar> : public std::string
 {
   SequenceToString(Glib::ustring::iterator pbegin, Glib::ustring::iterator pend);
 };
 
 template <>
-struct ustring::SequenceToString<Glib::ustring::const_iterator,gunichar> : public std::string
+struct ustring::SequenceToString<Glib::ustring::const_iterator, gunichar> : public std::string
 {
   SequenceToString(Glib::ustring::const_iterator pbegin, Glib::ustring::const_iterator pend);
 };
@@ -635,24 +701,24 @@ ustring_Iterator<T>& ustring_Iterator<T>::operator++()
 template <class T> inline
 const ustring_Iterator<T> ustring_Iterator<T>::operator++(int)
 {
-  const ustring_Iterator<T> tmp (*this);
+  const ustring_Iterator<T> temp (*this);
   this->operator++();
-  return tmp;
+  return temp;
 }
 
 template <class T> inline
 ustring_Iterator<T>& ustring_Iterator<T>::operator--()
 {
-  do { --pos_; } while((*pos_ & '\xC0') == '\x80');
+  do --pos_; while((*pos_ & '\xC0') == '\x80');
   return *this;
 }
 
 template <class T> inline
 const ustring_Iterator<T> ustring_Iterator<T>::operator--(int)
 {
-  const ustring_Iterator<T> tmp (*this);
+  const ustring_Iterator<T> temp (*this);
   this->operator--();
-  return tmp;
+  return temp;
 }
 
 #endif /* DOXYGEN_SHOULD_SKIP_THIS */
@@ -878,31 +944,31 @@ inline bool operator>=(const char* lhs, const ustring& rhs)
 
 /** @relates Glib::ustring */
 inline ustring operator+(const ustring& lhs, const ustring& rhs)
-  { return ustring(lhs) += rhs; }
+  { ustring temp (lhs); temp += rhs; return temp; }
 
 /** @relates Glib::ustring */
 inline ustring operator+(const ustring& lhs, const char* rhs)
-  { return ustring(lhs) += rhs; }
+  { ustring temp (lhs); temp += rhs; return temp; }
 
 /** @relates Glib::ustring */
 inline ustring operator+(const char* lhs, const ustring& rhs)
-  { return ustring(lhs) += rhs; }
+  { ustring temp (lhs); temp += rhs; return temp; }
 
 /** @relates Glib::ustring */
 inline ustring operator+(const ustring& lhs, gunichar rhs)
-  { return ustring(lhs) += rhs; }
+  { ustring temp (lhs); temp += rhs; return temp; }
 
 /** @relates Glib::ustring */
 inline ustring operator+(gunichar lhs, const ustring& rhs)
-  { return ustring(1, lhs) += rhs; }
+  { ustring temp (1, lhs); temp += rhs; return temp; }
 
 /** @relates Glib::ustring */
 inline ustring operator+(const ustring& lhs, char rhs)
-  { return ustring(lhs) += rhs; }
+  { ustring temp (lhs); temp += rhs; return temp; }
 
 /** @relates Glib::ustring */
 inline ustring operator+(char lhs, const ustring& rhs)
-  { return ustring(1, lhs) += rhs; }
+  { ustring temp (1, lhs); temp += rhs; return temp; }
 
 } // namespace Glib
 
